/*
 * mibdump.c --
 *
 *      Dump a MIB contents conforming to a given format.
 *
 * Copyright (c) 2002 Frank Strauss, Technical University of Braunschweig.
 * Copyright (c) 2002 Torsten Klie, Technical University of Braunschweig.
 *
 * See the file "COPYING" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * @(#) $Id: mibdump.c 1714 2002-11-19 13:12:40Z tklie $
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#ifdef HAVE_WIN_H
#include "win.h"
#endif

#include <smi.h>
#include "shhopt.h"
#include "mibdump.h"

#define MD_DEFAULT_SNMP_PORT       161
#define MD_DEFAULT_SNMP_HOST      "localhost"
#define MD_DEFAULT_SNMP_COMMUNITY "public"

static void help(void);
static void usage(void);
static void version(void);
static void port(unsigned int port);
static void community(char *community);
static void silent(void);
static void preload(char *module);
static void unified(void);
static void format(char *form);


static int flags;
static MibdumpDriver *driver;
static MibdumpDriver *firstDriver;
static MibdumpDriver *lastDriver;
static MibdumpDriver *defaultDriver;

static int opts;
static optStruct *opt;
static optStruct genericOpt[] = {
    /* short long              type        var/func       special       */
    { 'h', "help",           OPT_FLAG,   help,          OPT_CALLFUNC },
    { 'V', "version",        OPT_FLAG,   version,       OPT_CALLFUNC },
    { 'c', "community",      OPT_STRING, community,     OPT_CALLFUNC },
    { 's', "silent",         OPT_FLAG,   silent,        OPT_CALLFUNC },
    { 'p', "port",           OPT_UINT,   port,          OPT_CALLFUNC },
    { 'P', "preload",        OPT_STRING, preload,       OPT_CALLFUNC },
    { 'u', "unified",        OPT_FLAG,   unified,       OPT_CALLFUNC },
    { 'f', "format",         OPT_STRING, format,        OPT_CALLFUNC },
    { 0, 0, OPT_END, 0, 0 }  /* no more options */
};



void *xmalloc(size_t size)
{
    char *m = malloc(size);
    if (! m) {
	fprintf(stderr, "mibdump: malloc failed - running out of memory\n");
	exit(1);
    }
    return m;
}



void *xrealloc(void *ptr, size_t size)
{
    char *m = realloc(ptr, size);
    if (! m) {
	fprintf(stderr, "mibdump: realloc failed - running out of memory\n");
	exit(1);
    }
    return m;
}



char *xstrdup(const char *s)
{
    char *m = strdup(s);
    if (! m) {
	fprintf(stderr, "mibdump: strdup failed - running out of memory\n");
	exit(1);
    }
    return m;
}



void xfree(void *ptr)
{
    free(ptr);
}



void mibdumpRegisterDriver(MibdumpDriver *driver)
{
    int i;
    
    if (!firstDriver) {
	firstDriver = driver;
	lastDriver = driver;
    } else {
	lastDriver->next = driver;
	lastDriver = driver;
    }

    for (i = 0; driver->opt && driver->opt[i].type != OPT_END; i++) {
	opt = xrealloc(opt, (opts+1) * sizeof(optStruct));
	memcpy(&opt[opts], &opt[opts-1], sizeof(optStruct));
	opt[opts-1].shortName = 0;
	opt[opts-1].longName  = xmalloc(strlen(driver->name) +
				 strlen(driver->opt[i].name) + 2);
	sprintf(opt[opts-1].longName, "%s-%s",
		driver->name, driver->opt[i].name);
	opt[opts-1].type      = driver->opt[i].type;
	opt[opts-1].arg       = driver->opt[i].arg;
	opt[opts-1].flags     = driver->opt[i].flags;
	opts++;
    }
}



static void formats()
{
    MibdumpDriver *driver = firstDriver;
    
    for (driver = firstDriver; driver; driver = driver->next) {
	fprintf(stderr, "  %-14s: %s\n", driver->name,
 		driver->descr ? driver->descr : "...");
    }
}



static void usage()
{
    int i;
    MibdumpDriver *driver;
    char *value = NULL;
    
    fprintf(stderr,
	    "Usage: mibdump [options] host module\n"
	    "  -V, --version          show version and license information\n"
	    "  -h, --help             show usage information\n"
	    "  -s, --silent           do not generate any comments\n"
	    "  -p, --port=port        use port <port> (default %d)\n"
	    "  -c, --community=name   snmp community string (default %s)\n"
	    "  -f, --format=format    use <format> when dumping (default %s)\n"
	    "  -u, --unified          print a single unified output of all modules\n\n",
	    MD_DEFAULT_SNMP_PORT, MD_DEFAULT_SNMP_COMMUNITY,
	    defaultDriver ? defaultDriver->name : "none");

    fprintf(stderr, "Supported formats are:\n");
    formats();

    for (driver = firstDriver; driver; driver = driver->next) {
	if (! driver->opt) continue;
	fprintf(stderr, "\nSpecific option for the \"%s\" format:\n",
		driver->name);
	for (i = 0; driver->opt && driver->opt[i].type != OPT_END; i++) {
	    int n;
	    switch (driver->opt[i].type) {
	    case OPT_END:
	    case OPT_FLAG:
		value = NULL;
		break;
	    case OPT_STRING:
		value = "string";
		break;
	    case OPT_INT:
	    case OPT_UINT:
	    case OPT_LONG:
	    case OPT_ULONG:
		value = "number";
		break;
	    }
	    fprintf(stderr, "  --%s-%s%s%s%n",
		    driver->name, driver->opt[i].name,
		    value ? "=" : "",
		    value ? value : "",
		    &n);
	    fprintf(stderr, "%*s%s\n",
		    30-n, "",
		    driver->opt[i].descr ? driver->opt[i].descr : "...");
	}
    }
}



static void help() { usage(); exit(0); }
static void version() { printf("mibdump " SMI_VERSION_STRING "\n"); exit(0); }
static void port( unsigned int port ) { snmpPort = port; }
static void community( char *community ) { snmpCommunity = community; }
static void silent() { flags |= MIBDUMP_FLAG_SILENT; }
static void preload(char *module) { smiLoadModule(module); }
static void unified() { flags |= MIBDUMP_FLAG_UNITE; }

static void format(char *form)
{
    for (driver = firstDriver; driver; driver = driver->next) {
	if (strcasecmp(driver->name, form) == 0) {
	    break;
	}
    }
    if (!driver) {
	fprintf(stderr, "mibdump: invalid dump format `%s'"
		" - supported formats are:\n", form);
	formats();
	exit(1);
    }
}



int main(int argc, char *argv[])
{
    char *modulename;
    SmiModule *smiModule;
    int smiflags;

    firstDriver = lastDriver = defaultDriver = NULL;

    snmpHost = NULL;  snmpCommunity = NULL;
    snmpPort = MD_DEFAULT_SNMP_PORT;
    
    opts = sizeof(genericOpt) / sizeof(optStruct);
    opt = xmalloc(sizeof(genericOpt));
    memcpy(opt, genericOpt, sizeof(genericOpt));

  
    initXml();
    defaultDriver = lastDriver;

#if 0
    for (i = 1; i < argc; i++)
	if ((strstr(argv[i], "-c") == argv[i]) ||
	    (strstr(argv[i], "--config") == argv[i])) break;
    if (i == argc) 
	smiInit("mibdump");
    else
	smiInit(NULL);
#endif /* #if 0 */

    smiInit( "mibdump" );
    smiSetErrorLevel( 0 );
    
    flags = 0;
    driver = defaultDriver;

    optParseOptions(&argc, argv, opt, 0);

    if( argv[1] ) {
	snmpHost = argv[1];
    }
    if( !snmpCommunity ) {
	snmpCommunity = MD_DEFAULT_SNMP_COMMUNITY;
    }
    
    if (!driver) {
	fprintf(stderr, "mibdump: no dump formats registered\n");
	smiExit();
	exit(1);
    }
    
    smiflags = smiGetFlags();
    smiflags |= SMI_FLAG_ERRORS;
    smiflags |= driver->smiflags;
    smiSetFlags(smiflags);

    if (flags & MIBDUMP_FLAG_UNITE && driver->ignflags & MIBDUMP_DRIVER_CANT_UNITE) {
	fprintf(stderr, "mibdump: %s format does not support united output:"
		" ignoring -u\n", driver->name);
	flags = (flags & ~MIBDUMP_FLAG_UNITE);
    }

    if( argc != 3 ) {
      usage();
      smiExit();
      exit(0);
    }
    
    modulename = smiLoadModule( argv[ 2 ] );
    smiModule = modulename ? smiGetModule( modulename ) : NULL;
    if( smiModule ) {
	if( smiModule->conformance < 3 ) {
	    if (! (flags & MIBDUMP_FLAG_SILENT)) {
		fprintf(stderr,
			"mibdump: module `%s' contains errors, "
			"expect flawed output\n",
			argv[2]);
	    }
	}
    }
    else {
	fprintf(stderr, "mibdump: cannot locate module `%s'\n",
		argv[ 2 ]);
	exit(2);
    }

    (driver->func)( smiModule, flags );

    smiExit();

    return 0;
}
